/**
    @file ft51_i2c_slave.c

    @brief I2C Slave Library API functions.

    @version 1.0.0

**/
/*
 * ============================================================================
 * History
 * =======
 *
 * (C) Copyright 2014, Future Technology Devices International Ltd.
 * ============================================================================
 *
 * This source code ("the Software") is provided by Future Technology Devices
 * International Limited ("FTDI") subject to the licence terms set out
 * http://www.ftdichip.com/FTSourceCodeLicenceTerms.htm ("the Licence Terms").
 * You must read the Licence Terms before downloading or using the Software.
 * By installing or using the Software you agree to the Licence Terms. If you
 * do not agree to the Licence Terms then do not download or use the Software.
 *
 * Without prejudice to the Licence Terms, here is a summary of some of the key
 * terms of the Licence Terms (and in the event of any conflict between this
 * summary and the Licence Terms then the text of the Licence Terms will
 * prevail).
 *
 * The Software is provided "as is".
 * There are no warranties (or similar) in relation to the quality of the
 * Software. You use it at your own risk.
 * The Software should not be used in, or for, any medical device, system or
 * appliance. There are exclusions of FTDI liability for certain types of loss
 * such as: special loss or damage; incidental loss or damage; indirect or
 * consequential loss or damage; loss of income; loss of business; loss of
 * profits; loss of revenue; loss of contracts; business interruption; loss of
 * the use of money or anticipated savings; loss of information; loss of
 * opportunity; loss of goodwill or reputation; and/or loss of, damage to or
 * corruption of data.
 * There is a monetary cap on FTDI's liability.
 * The Software may have subsequently been amended by another user and then
 * distributed by that other user ("Adapted Software").  If so that user may
 * have additional licence terms that apply to those amendments. However, FTDI
 * has no liability in relation to those amendments.
 * ============================================================================
 */
#include <stdint.h>
#include "ft51.h"
#include "registers/ft51_registers.h"

// I2C #defines
#define I2C_STATUS_ACTIVE       0x10
#define I2C_STATUS_RECFIN       0x08
#define I2C_STATUS_SENDFIN      0x04
#define I2C_STATUS_TREQ         0x02
#define I2C_STATUS_RREQ         0x01
#define I2C_FLAGS_RECFIN        0x08
#define I2C_FLAGS_SENDFIN       0x04
#define I2C_FLAGS_DA            0x40
#define I2C_FLAGS_RSTB          0x80

void I2C_slave_initialise(uint8_t addr)
{
    // Reset then enable I2C Slave block
    I2CSCR |= I2C_FLAGS_RSTB;
    __asm__ ("NOP");

    I2CSCR = 0;
    __asm__ ("NOP");

    I2CSCR = I2C_FLAGS_RECFIN | I2C_FLAGS_SENDFIN | I2C_FLAGS_DA;
    __asm__ ("NOP");

    // Slave address
    I2CSOA = addr;
}

uint16_t I2C_slave_read(uint8_t *data, uint16_t number_to_read)
{
    uint8_t ssr;
    uint16_t number_read = 0;

    while (number_to_read--)
    {
        ssr = I2CSSR;
        if (ssr & I2C_STATUS_RREQ)
        {
            *data++ = I2CSBUF;
            number_read++;
        }
        if (ssr & I2C_STATUS_RECFIN)
        {
            I2CSCR |= I2C_FLAGS_RECFIN;
            break;
        }
    };

    return number_read;
}

uint16_t I2C_slave_write(const uint8_t *data, uint16_t number_to_write)
{
    uint8_t ssr;
    uint16_t number_written = 0;

    while (number_to_write--)
    {
        ssr = I2CSSR;
        if (ssr & I2C_STATUS_TREQ)
        {
            I2CSBUF = *data++;
            number_written++;
        }
        if (ssr & I2C_STATUS_SENDFIN)
        {
            I2CSCR |= I2C_FLAGS_SENDFIN;
            break;
        }
    };

    return number_written;
}

uint8_t I2C_slave_get_status(void)
{
    return I2CSSR;
}