/**
    @file ft51_usb_remote_wakeup_feature.c

    @brief USB Library API remote wakeup feature request handler functions.

    @version 1.0.0

**/
/*
 * ============================================================================
 * History
 * =======
 *
 * (C) Copyright 2014, Future Technology Devices International Ltd.
 * ============================================================================
 *
 * This source code ("the Software") is provided by Future Technology Devices
 * International Limited ("FTDI") subject to the licence terms set out
 * http://www.ftdichip.com/FTSourceCodeLicenceTerms.htm ("the Licence Terms").
 * You must read the Licence Terms before downloading or using the Software.
 * By installing or using the Software you agree to the Licence Terms. If you
 * do not agree to the Licence Terms then do not download or use the Software.
 *
 * Without prejudice to the Licence Terms, here is a summary of some of the key
 * terms of the Licence Terms (and in the event of any conflict between this
 * summary and the Licence Terms then the text of the Licence Terms will
 * prevail).
 *
 * The Software is provided "as is".
 * There are no warranties (or similar) in relation to the quality of the
 * Software. You use it at your own risk.
 * The Software should not be used in, or for, any medical device, system or
 * appliance. There are exclusions of FTDI liability for certain types of loss
 * such as: special loss or damage; incidental loss or damage; indirect or
 * consequential loss or damage; loss of income; loss of business; loss of
 * profits; loss of revenue; loss of contracts; business interruption; loss of
 * the use of money or anticipated savings; loss of information; loss of
 * opportunity; loss of goodwill or reputation; and/or loss of, damage to or
 * corruption of data.
 * There is a monetary cap on FTDI's liability.
 * The Software may have subsequently been amended by another user and then
 * distributed by that other user ("Adapted Software").  If so that user may
 * have additional licence terms that apply to those amendments. However, FTDI
 * has no liability in relation to those amendments.
 * ============================================================================
 */
#include <stdint.h>
#include <stdlib.h>
#include "ft51.h"
#include "ft51_sfr.h"
#include "registers/ft51_registers.h"
#include "ft51_usb.h"
#include "ft51_usb_internal.h"

uint8_t USB_get_remote_wakeup(void)
{
    uint8_t status;

    // Return a non-zero value if remote wakeup is enabled
    DISABLE_INTERRUPTS;
    status = (USB_hub_config & MASK_HUB_REMOTE_WAKEUP_EN);
    ENABLE_INTERRUPTS;

    return status;
}

void USB_set_remote_wakeup(void)
{
    DISABLE_INTERRUPTS;
    USB_hub_config |= MASK_HUB_REMOTE_WAKEUP_EN;
    ENABLE_INTERRUPTS;
}

void USB_clear_remote_wakeup(void)
{
    DISABLE_INTERRUPTS;
    USB_hub_config &= (~MASK_HUB_REMOTE_WAKEUP_EN);
    ENABLE_INTERRUPTS;
}

void USB_wakeup(void)
{
    if (USB_get_remote_wakeup())
    {
        FT122_CMD = SEND_RESUME;
    }
}